# This file was generated by generate_stubs.rb
# Do not edit this file directly.

# A FileDescriptor provides information about all Protobuf definitions in a
# particular file.
class Google::Protobuf::FileDescriptor

  # Returns a new file descriptor. May
  # to a builder.
  def initialize; end

  # Returns the name of the file.
  # 
  # @return [String]
  def name; end

  # Returns the
  # {https://github.com/protocolbuffers/protobuf/blob/v30.2/src/google/protobuf/descriptor.proto#L442
  # FileOptions} for this {FileDescriptor}.
  # 
  # @return [FileOptions]
  def options; end

  # Returns the
  # {https://github.com/protocolbuffers/protobuf/blob/v30.2/src/google/protobuf/descriptor.proto#L104
  # FileDescriptorProto} of this {FileDescriptor}.
  # 
  # @return [FileDescriptorProto]
  def to_proto; end

end # class Google::Protobuf::FileDescriptor
